#!/usr/bin/python
#---------------------------#
#---TESTING GUI FOR RYGEL---#
#---------------------------#
#   ONBOARD LED             |   CHECKBOX
#   PAN TILT SLIDERS        |   SLIDERS
#   DRIVE MOTOR SLIDERS     |   SLIDERS
#   SONAR PING BUTTON(s)    |   BUTTON/LABELS
#   GYRO/ACC BUTTON(s)      |   BUTTON/LABELS
#---------------------------#
from Tkinter import *
import serial
from time import sleep

root = Tk()
boardled_var = BooleanVar()
panservo_var = IntVar()
tiltservo_var = IntVar()
LFservo_var = IntVar()
RFservo_var = IntVar()
LBservo_var = IntVar()
RBservo_var = IntVar()
sonar0_text = StringVar()
sonar1_text = StringVar()
fimu_text = StringVar()

#------------------#
#---UTILITY FXNS---#
#------------------#
def set_boardled():  # TOGGLE BOARD LED
    boardled_val = boardled_var.get()
    print ":SET BOARD LED", boardled_val
    if boardled_val == 0: mySerial.write("LON\n")
    if boardled_val == 1: mySerial.write("LOFF\n")    

def set_panservo(value):  # SET PAN SERVO    
    atxt = "pan " + str(value) + '\n'
    mySerial.write(atxt)
    print ":SET PAN SERVO", value, atxt
def set_tiltservo(value):  # SET TILT SERVO
    atxt = "tilt " + str(value) + '\n'
    mySerial.write(atxt)    
    print ":SET TILT SERVO", value, atxt
    
def set_LFservo(value):  # SET MOTOR SERVO - LEFT FRONT
    atxt = "drive 1 " + str(value) + '\n'
    mySerial.write(atxt)
def set_RFservo(value):  # SET MOTOR SERVO - RIGHT FRONT
    atxt = "drive 2 " + str(value) + '\n'
    mySerial.write(atxt)
def set_LBservo(value):  # SET MOTOR SERVO - LEFT BACK
    atxt = "drive 3 " + str(value) + '\n'
    mySerial.write(atxt)
def set_RBservo(value):  # SET MOTOR SERVO - RIGHT FRONT
    atxt = "drive 4 " + str(value) + '\n'
    mySerial.write(atxt)    

def get_sonar0():  # GET SONAR RANGE - 0 
    mySerial.write("sping 0\n")
    tempvar = mySerial.readline()
    sonar0_text.set(tempvar)
def get_sonar1():  # GET SONAR RANGE - 1
    mySerial.write("sping 1\n")
    tempvar = mySerial.readline()
    sonar1_text.set(tempvar)   

def get_freeimu():  # GET GYRO/ACC VALUES
    mySerial.write("fimu\n")
    tempvar = mySerial.readline()
    fimu_text.set(tempvar)

#--------------#
#---GUI FXNS---#
#--------------#
def onClose():
    print ":::CLOSING TKGUI and SERIAL PORT"
    mySerial.close()
    root.destroy()
    print ":::GOODBYE!!!"

#------------------#
#---SETUP SERIAL---#
#------------------#
#mySerial_device = '/dev/ttyUSB0'
mySerial_device = '/dev/ttyUSB1'
#mySerial_baud = 115200
mySerial_baud = 9600
mySerial = serial.Serial(mySerial_device, mySerial_baud,
                         timeout=1)
#mySerial = serial.Serial(0)
mySerial.open()
if mySerial.isOpen():
    print mySerial
    print '|  -SERIAL SETUP COMPLETE'
else: print '|  -SERIAL SETUP FAILED'


#######################################3


#---------------#
#---SETUP GUI---#
#---------------#
#---PRESET GUI VARIABLES
panservo_var.set(90)
tiltservo_var.set(90)
LFservo_var.set(90)
RFservo_var.set(90)
LBservo_var.set(90)
RBservo_var.set(90)

#---BOARD LED
button_led = Checkbutton(root, text = "LED", command = set_boardled,
                variable = boardled_var)
button_led.pack(anchor = CENTER)

#---PAN/TILT
slider_pan = Scale(root, from_=0, to=180, command = set_panservo,
                variable = panservo_var, orient = HORIZONTAL)
slider_pan.pack()
slider_tilt = Scale(root, from_=0, to=180, command = set_tiltservo,
                variable = tiltservo_var, orient = VERTICAL)
slider_tilt.pack()

#---SONAR
button_sonar0 = Button(root, text="SONAR0", width=10, height=2,
                command = get_sonar0)
button_sonar0.pack()
label_sonar0 = Label(root, textvariable = sonar0_text)
label_sonar0.pack()
button_sonar1 = Button(root, text="SONAR1", width=10, height=2,
                command = get_sonar1)
button_sonar1.pack()
label_sonar1 = Label(root, textvariable = sonar1_text)
label_sonar1.pack()

#---DRIVE MOTORS
slider_driveLF = Scale(root, from_=0, to=180, command = set_LFservo,
                variable = LFservo_var, orient = HORIZONTAL)
slider_driveLF.pack()
slider_driveRF = Scale(root, from_=0, to=180, command = set_RFservo,
                variable = RFservo_var, orient = HORIZONTAL)
slider_driveRF.pack()
slider_driveLB = Scale(root, from_=0, to=180, command = set_LBservo,
                variable = LBservo_var, orient = HORIZONTAL)
slider_driveLB.pack()
slider_driveRB = Scale(root, from_=0, to=180, command = set_RBservo,
                variable = RBservo_var, orient = HORIZONTAL)
slider_driveRB.pack()

#---GYRO/ACC
button_fimu = Button(root, text="FREEIMU", width=10, height=2,
                command = get_freeimu)
button_fimu.pack()
label_fimu = Label(root, textvariable = fimu_text)
label_fimu.pack()

#---GUI LOOP
root.protocol("WM_DELETE_WINDOW", onClose)
root.mainloop()



"""
//------------------------------NOTES--------------------------------//

DRIVE MOTORS | 4=RB(-) | 3=LB(-) | 2=RF(-) | 1=LF
0-89 REVERSE / 91-180-FORWARD

LON     TURN BOARD LED ON
LOFF",    LED_off);       //-TURN BOARD LED OFF
pan | angle | delay
tile | angle
drive | motor (1-4) | speed (0-180)
sping | sonar# (0,1) | {returns cm}
fimu | {returns raw vals}

"""






