//-------------------------------------------------------------------//
//------------------------------NEXT---------------------------------//
//-------------------------------------------------------------------//
//---TEST WITH Pyserial / command line serial.
//  POWER ISSUES--- 6 servos...
//  ASSEMBLE
//--------------------------NOTES TO KEEP----------------------------//
//-SONAR = HC-SR04
//-GYRO = MPU6050 | GY-521
//-PAN/TILT SERVOS | 9g angle (0-180?ish)
//-MOTOR SERVOS | Xg continuous rotation (0-180?ish)
//-------------------------------------------------------------------//
#include <SerialCommand.h>
#include <Servo.h>
#include <NewPing.h>

#define SERIAL_SPEED   9600
#define arduinoLED     13  //---ONBOARD LEAD
#define TRIGGER_PIN_0  14  //---SONAR 0 TRIGGER
#define ECHO_PIN_0     15  //---SONAR 0 ECHO
#define TRIGGER_PIN_1  16  //---SONAR 1 TRIGGER
#define ECHO_PIN_1     17  //---SONAR 1 ECHO
#define MAX_DISTANCE   200 //---SONAR MAX DISTANCE
#define pinPANmotor    10  //---PAN MOTOR PIN
#define pinTILTmotor   11  //---TILT MOTOR PIN
#define pinLFmotor     3   //---LEFT FRONT MOTOR PIN
#define pinRFmotor     5   //---RIGHT FRONT MOTOR PIN
#define pinLBmotor     6   //---LEFT BACK MOTOR PIN
#define pinRBmotor     9   //---RIGHT BACK MOTOR PIN

//---SERVO MIN/MAX VARS
int posPANmin = 60; int posPANmax = 120;
int posTILTmin = 40; int posTILTmax = 50;
int posMOTORmin = 85; int posMOTORmax = 95;
//---SERVO POSITION VARS
int posPANmotor = posPANmin + ((posPANmax - posPANmin) / 2);      //---DEFAULT PAN
//int posTILTmotor = (posTILTmax - posTILTmin) / 2;   //---DEFAULT TILT
int posTILTmotor = 40;
int posLFmotor = (posMOTORmax - posMOTORmin) / 2;  //---LEFT FRONT
int posRFmotor = (posMOTORmax - posMOTORmin) / 2;  //---RIGHT FRONT
int posRBmotor = (posMOTORmax - posMOTORmin) / 2;  //---LEFT BACK
int posLBmotor = (posMOTORmax - posMOTORmin) / 2;  //---RIGHT BACK
//---SERVO MOTOR OBJECTS
Servo objPANmotor;
Servo objTILTmotor;
Servo objLFmotor;
Servo objRFmotor;
Servo objLBmotor;
Servo objRBmotor;
//---SONAR PING OBJECTS
NewPing sonar0(TRIGGER_PIN_0, ECHO_PIN_0, MAX_DISTANCE);
NewPing sonar1(TRIGGER_PIN_1, ECHO_PIN_1, MAX_DISTANCE);
//--SERIAL COMMAND OBJECT
SerialCommand sCmd;

//--------------------------//
//---SETUP/LOOP FUNCTIONS---//
//--------------------------//
void setup() {
  pinMode(arduinoLED, OUTPUT);    //-SETUP ONBOARD LED
  digitalWrite(arduinoLED, LOW);  //-SET ONBOARD LED OFF

  objPANmotor.attach(pinPANmotor);  //-ATTACH SERVO MOTOR OBJECTS
  //objTILTmotor.attach(pinTILTmotor);
  objLFmotor.attach(pinLFmotor);
  objRFmotor.attach(pinRFmotor);
  objLBmotor.attach(pinLBmotor);
  objRBmotor.attach(pinRBmotor);  

  //objPANmotor.write(posPANmotor);  //-SET PAN/TILT TO DEFAULT POSITIONS
  //objTILTmotor.write(posTILTmotor);

  Serial.begin(SERIAL_SPEED);
  //---SETUP CALLBACKS FOR SerialCommand
  sCmd.addCommand("LON",    LED_on);         //-TURN BOARD LED ON
  sCmd.addCommand("LOFF",    LED_off);       //-TURN BOARD LED OFF
  sCmd.addCommand("pan",   PANmotor_sweep);        //-pan | angle | delay
  sCmd.addCommand("tilt",   TILTmotor_sweep);      //-tile | angle
  sCmd.addCommand("drive",  DRIVEmotor);     //-drive | motor (0-3|LF,RF,LB,RB) | angle/speed
  sCmd.addCommand("sping", SONARping);       //-sping | sonar# (0,1) {returns cm}
  sCmd.setDefaultHandler(unrecognized);      //-HANDLER FOR UNKNOWN COMMANDS
  Serial.println("Ready");
}

void loop() {
  sCmd.readSerial();  //---PROCESS SERIAL COMMANDS
}

//-----------------------//
//---UTILITY FUNCTIONS---//
//-----------------------//
//---CHECK IF POSITIVE OR NEGATIVE - RETURNS -1/1 if <1/>1
int negpos(int invar){
  int outvar;
  if (invar >= 0){outvar = 1;}
  else if (invar < 0){outvar = -1;}
  return outvar;  
}

//---ULTRASONIC PING - RETURNS DISTANCE IN CENTIMETERS
void SONARping() {
  int aNumber; char *arg;
  unsigned int uS = 999;
  arg = sCmd.next();
  if (arg != NULL) {
    aNumber = atoi(arg);
    if (aNumber == 0) {uS = sonar0.ping();} //---SONAR 0
    if (aNumber == 1) {uS = sonar1.ping();} //---SONAR 1
  Serial.println(uS / US_ROUNDTRIP_CM);
  }
}

//---SET CONTINUOUS ROTATION SERVO MOTOR SPEED - motor# | angle
void DRIVEmotor() {
  int aMotor; int aAngle;
  char *arg;
  arg = sCmd.next(); //-SERVO NUMBER | 0=LF, 1=RF, 2=LB, 3=RB
  if (arg != NULL){aMotor = atoi(arg);}  
  arg = sCmd.next(); //-ANGLE TO SET (+/-ANGLE = SPEED)
  if (arg != NULL){aAngle = min(max(atoi(arg),posPANmin),posPANmax);}
  //int aAngle = min(max(atoi(arg),posPANmin),posPANmax);      
  if (aMotor != NULL && aAngle != NULL) {
    if (aMotor == 0){objLFmotor.write(aAngle);}
    if (aMotor == 1){objRFmotor.write(aAngle);}
    if (aMotor == 2){objLBmotor.write(aAngle);}
    if (aMotor == 3){objRBmotor.write(aAngle);}
    Serial.print(":::DRIVE MOTOR - ");
    Serial.print(aMotor);
    Serial.print(" - TO - ");    
    Serial.println(aAngle);
  }
}

//---SWEEPS PAN MOTOR, CHECKS MIN/MAX VALUES - newangle | delay
//---later maybe do a quadratic acceleration to new angle?
void PANmotor_sweep() {
  int tempDelay = 10;
  int aNewAngle; //int aDelay;
  char *arg;
  arg = sCmd.next(); //-SERVO NUMBER | 0=LF, 1=RF, 2=LB, 3=RB
  if (arg != NULL){
    aNewAngle = min(max(atoi(arg),posPANmin),posPANmax);
    int angleDifference = aNewAngle - posPANmotor;
    int np = negpos(angleDifference);
    if (np > 0){
        for(int ang = posPANmotor; ang <= aNewAngle; ang++){        
        objPANmotor.write(ang);
        delay(tempDelay);
      }
    }
    if (np < 0){
        for(int ang = posPANmotor; ang >= aNewAngle; ang--){        
        objPANmotor.write(ang);
        delay(tempDelay);
      }
    }
    posPANmotor = aNewAngle;
  }
}

//---SWEEPS TILT MOTOR, CHECKS MIN/MAX VALUES - newangle | delay
//---later maybe do a quadratic acceleration to new angle?
void TILTmotor_sweep() {
  int tempDelay = 10;
  int aNewAngle; //int aDelay;
  char *arg;
  arg = sCmd.next(); //-SERVO NUMBER | 0=LF, 1=RF, 2=LB, 3=RB
  if (arg != NULL){
    aNewAngle = min(max(atoi(arg),posTILTmin),posTILTmax);
    int angleDifference = aNewAngle - posTILTmotor;
    int np = negpos(angleDifference);
    if (np > 0){
        for(int ang = posTILTmotor; ang <= aNewAngle; ang++){        
        objTILTmotor.write(ang);
        delay(tempDelay);
      }
    }
    if (np < 0){
        for(int ang = posTILTmotor; ang >= aNewAngle; ang--){        
        objTILTmotor.write(ang);
        delay(tempDelay);
      }
    }
    posTILTmotor = aNewAngle;
  }
}

void TILTmotor_old() {
  int aNumber; char *arg;
  arg = sCmd.next();
  if (arg != NULL) {
    aNumber = min(max(atoi(arg),posTILTmin),posTILTmax);
    objTILTmotor.write(aNumber);
    Serial.print(":::TILT TO - ");
    Serial.println(aNumber);    
  }
}

void LED_on() {
  Serial.println("::BOARD LED ON");
  digitalWrite(arduinoLED, HIGH);
}

void LED_off() {
  Serial.println(":::BOARD LED OFF");
  digitalWrite(arduinoLED, LOW);
}

//---HANDLES UNKNOWN SerialCommands
void unrecognized(const char *command) {
  Serial.println("Do what now?");
}


