/*
|----------------------------------------|
 |------------BLUE BOX ARDUINO------------|
 |----------------------------------------|
 WS2811 LED STRIP
 HC-SR04 ULTRASONIC RANGEFINDER
 SOUND DETECTION MODULE
 DS18S20 DIGITAL TEMPERATURE SENSOR
 |----------------------------------------|
 -get all components tested and working at least
 -add color to VU, GREEN, YELLOW, RED
 -ADD - RF and InfraRed TX/RX w/ software serial
 -BT NOT WORKING...
 -PAN/TILT SWEEP is WONKY
 
 http://wiibrew.org/wiki/Wiimote/Extension_Controllers
 |----------------------------------------|
  Just plug it into the Wii Nunchuck, then into the Arduino, in to Analog In pins 2,3,4,5. Pins 4&5 are the I2C communication pins and Pins 2&3 can act as the power supply for the Nunchuck.
 */

#include <FastSPI_LED.h>
#include <OneWire.h>
#include <DallasTemperature.h>
#include <NewPing.h>
#include <Servo.h>
#include <SoftwareSerial.h>
#include "Wire.h"
#include "WiiChuck.h"

//---PIN CONSTANTS
#define PIN_BUTTON_A    0    //---DIGITAL PIN 2 IS INTERRUPT 0 - WHY?
#define PIN_MIC_DIGI    12
#define PIN_MIC_ANLO    A7
#define PIN_PING_TRIG   A1
#define PIN_PING_ECHO   A0
#define PIN_THERMO      8
#define PIN_LED_STRIP   3
#define PIN_SERVO_PAN   10
#define PIN_SERVO_TILT  9
#define PIN_LASER       6
#define PIN_SOFT_RX     11   //---TO BT TX  //---A/B/C/D | Rx/Tx/Gd/Vc | 5/4/G/V
#define PIN_SOFT_TX     7    //---TO BT RX
// * RX is digital pin 11 (connect to TX of other device)
// * TX is digital pin 11 (connect to RX of other device)
//---WIICHUCK ON 2,3,4,5

//---OTHER CONSTANTS
#define NUM_LEDS          14    //---NUMBER OF LEDS ON STRIP
#define LED_DATA_RATE     2     //---LED DATA RATE
#define MAX_PING_DISTANCE 60    //---MAX PING DISTANCE IN CM
#define MIN_TEMPERATURE   10    //---MIN TEMPERATURE (FOR VU DISPLAY)
#define MAX_TEMPERATURE   60    //---MAX TEMPERATURE (FOR VU DISPLAY)
#define TEMP_PRECISION    9     //---TEMPERATURE PRECISION (9-12?) - NOT USED RIGHT NOW...
#define MAX_AUDIO_LEVEL   1000  //---MAX AUDIO LEVEL (FOR VU DISPLAY)
//#define AUDIO_MULTIPLIER  10
#define MAX_MODE          9
//#define BUTTON_DOWN_STATE LOW //---STATE BUTTON IS IN WHEN PRESSED
#define MIN_PAN_ANGLE     60
#define MAX_PAN_ANGLE     120
#define MIN_TILT_ANGLE    60
#define MAX_TILT_ANGLE    120



//---VARIABLES / ARRAYS
struct CRGB { 
  unsigned char b; 
  unsigned char r; 
  unsigned char g; 
}; //---LED STRIP VAR
struct CRGB *led_array;
int idex = 0;            //---LED INDEX (0 to NUM_LEDS-1
int ihue = 0;            //---HUE (0-360)
int bouncedirection = 0; //---SWITCH FOR COLOR BOUNCE (0-1)
int servoPANpos = 90;     //---SERVO PAN POSITION VAR
int servoTILTpos = 90;    //---SERVO TILT POSITION VAR
int laserPOWER = 0;      //---LASER POWER VAR (0-255)
volatile int thisMode = 0;
byte inbyte;  //---SERIAL INPUT BYTE
int thisarg;  //---SERIAL INPUT ARG
int gotserial = 0; //---INDICATES IF SERIAL BYTE RECIEVED

//---CREATE THERMOMETER OBJECT
OneWire oneWire(PIN_THERMO);
DallasTemperature sensor_thermo(&oneWire);

//---CREATE ULTRASONIC OBJECT
NewPing sonar(PIN_PING_TRIG, PIN_PING_ECHO, MAX_PING_DISTANCE);

//---CREATE SERVO OBJECT
Servo myservoPAN;
Servo myservoTILT;

//---CREATE SOFTWARE SERIAL COMMAND OBJECT
SoftwareSerial softSerial(PIN_SOFT_RX, PIN_SOFT_TX);

//---CREATE WIICHUCK OBJECT
WiiChuck chuck = WiiChuck();

//---------------------------------------------------------//
//---------------UTILITY/DISPLAY FUNCTIONS-----------------//
//---------------------------------------------------------//
int negpos(int invar){
  //---CHECK IF POSITIVE OR NEGATIVE - RETURNS -1/1 if <1/>1  
  int outvar;
  if (invar >= 0){
    outvar = 1;
  }
  else if (invar < 0){
    outvar = -1;
  }
  return outvar;  
}

void interruptHandler() {
  //---DON'T USE SERIAL IN AN INTERRUPT HANDLER!!!
  static unsigned long last_interrupt_time = 0;
  unsigned long interrupt_time = millis();
  if (interrupt_time - last_interrupt_time > 300) {
    thisMode++;
    if (thisMode > MAX_MODE) {
      thisMode = 0;
    }
    delay(50);
  }
  last_interrupt_time = interrupt_time;
}

//---INPUT VAR AND MAX, OUTPUT IN 0.0->1.0
float preVU(int in, int minof, int maxof) {
  float diffof = maxof - minof;
  float out = float(in-minof) / float(diffof);
  return out;
}

void VUThermo() {
  sensor_thermo.requestTemperatures();
  float this_temp = sensor_thermo.getTempCByIndex(0);
  float vu_temp = preVU(this_temp, MIN_TEMPERATURE, MAX_TEMPERATURE);
  LED_VUmeter(vu_temp);
  Serial.print("|---VUThermo: ");  
  Serial.print(this_temp);
  Serial.print(" - ");
  Serial.println(vu_temp);
  delay(50);
}

void VUPing() {
  unsigned int uS = sonar.ping();
  int cm_uS = uS / US_ROUNDTRIP_CM;
  float VU_uS = preVU(cm_uS, 0, MAX_PING_DISTANCE);
  LED_VUmeter(VU_uS);
  Serial.println("|---VUPing");  
  Serial.print(cm_uS);
  Serial.print(" - ");
  Serial.println(VU_uS);
  delay(50);  
}

void VUAudio() {
  int this_audio_a = analogRead(PIN_MIC_ANLO);
  int this_audio_d = digitalRead(PIN_MIC_DIGI);
  float vu_audio = preVU(this_audio_a, 0, MAX_AUDIO_LEVEL);
  LED_VUmeter(vu_audio);
  Serial.print("|---VUAudio: ");
  Serial.print(this_audio_a);
  Serial.print(" - ");  
  Serial.print(vu_audio);
  Serial.print(" - ");    
  Serial.println(this_audio_d);
  delay(50);  
}



//---------------------------------------------------------//
//---------------------LED FUNCTIONS-----------------------//
//---------------------------------------------------------//
//---LED VU METER THING - INPUT 0.0 - 1.0
void LED_VUmeter(float this_level) {
  int full_on = NUM_LEDS*255; //=14 * 255;
  int level_on = full_on * this_level;
  int leds_full = level_on / 255;
  int led_remainder = level_on % 255;
  for(int i = 0; i < leds_full ; i++ ) {
    led_array[NUM_LEDS-1-i].r = 255;
    led_array[NUM_LEDS-1-i].g = 0;
    led_array[NUM_LEDS-1-i].b = 0; 
  }
  for(int i = leds_full; i < NUM_LEDS ; i++ ) {
    led_array[NUM_LEDS-1-i].r = 0;
    led_array[NUM_LEDS-1-i].g = 0;
    led_array[NUM_LEDS-1-i].b = 0; 
  }  
  if (leds_full < NUM_LEDS) {
    set_color_led(leds_full, led_remainder, 0, 0);
  }
  FastSPI_LED.show();
}

//---SET THE COLOR OF A SINGLE RGB LED
void set_color_led(int adex, int cred, int cgrn, int cblu) {  
  led_array[NUM_LEDS-1-adex].r = cred;
  led_array[NUM_LEDS-1-adex].g = cgrn;
  led_array[NUM_LEDS-1-adex].b = cblu;  
}

//---SET ALL LED'S TO ONE COLOR
void one_color_all(int cred, int cgrn, int cblu) { //-SET ALL LEDS TO ONE COLOR
  for(int i = 0 ; i < NUM_LEDS; i++ ) {
    set_color_led(i, cred, cgrn, cblu);
    FastSPI_LED.show(); 
    delay(1);
  }  
}

void rainbow_loop(int istep, int idelay) { //-LOOP HSV RAINBOW
  idex++;
  ihue = ihue + istep;
  int icolor[3];
  if (idex >= NUM_LEDS) {
    idex = 0;
  }
  if (ihue >= 359) {
    ihue = 0;
  }
  HSVtoRGB(ihue, 255, 255, icolor);
  set_color_led(idex, icolor[0], icolor[1], icolor[2]);
  FastSPI_LED.show();
  delay(idelay);
}

void color_bounce(int idelay) { //-BOUNCE COLOR (SINGLE LED)
  if (bouncedirection == 0) {
    idex = idex + 1;
    if (idex == NUM_LEDS) {
      bouncedirection = 1;
      idex = idex - 1;
    }
  }
  if (bouncedirection == 1) {
    idex = idex - 1;
    if (idex == 0) {
      bouncedirection = 0;
    }
  }  
  for(int i = 0; i < NUM_LEDS; i++ ) {
    if (i == idex) {
      set_color_led(i, 255, 0, 0);
    }
    else {
      set_color_led(i, 0, 0, 0);
    }
  }
  FastSPI_LED.show();  
  delay(idelay);
}

//-CONVERT HSV VALUE TO RGB
void HSVtoRGB(int hue, int sat, int val, int colors[3]) {
  // hue: 0-359, sat: 0-255, val (lightness): 0-255
  int r, g, b, base;

  if (sat == 0) { // Achromatic color (gray).
    colors[0]=val;
    colors[1]=val;
    colors[2]=val;
  } 
  else  {
    base = ((255 - sat) * val)>>8;
    switch(hue/60) {
    case 0:
      r = val;
      g = (((val-base)*hue)/60)+base;
      b = base;
      break;
    case 1:
      r = (((val-base)*(60-(hue%60)))/60)+base;
      g = val;
      b = base;
      break;
    case 2:
      r = base;
      g = val;
      b = (((val-base)*(hue%60))/60)+base;
      break;
    case 3:
      r = base;
      g = (((val-base)*(60-(hue%60)))/60)+base;
      b = val;
      break;
    case 4:
      r = (((val-base)*(hue%60))/60)+base;
      g = base;
      b = val;
      break;
    case 5:
      r = val;
      g = base;
      b = (((val-base)*(60-(hue%60)))/60)+base;
      break;
    }
    colors[0]=r;
    colors[1]=g;
    colors[2]=b;
  }
}

void servo_sine_circle() {
  for(int i = 0; i < 180; i += 1) {
    float pval = sin(radians(i));
    float tval = cos(radians(i));
    int pang = 180 * pval;
    int tang = 180 * tval;
    int servo_pang = min(max(pang, MIN_PAN_ANGLE), MAX_PAN_ANGLE);
    int servo_tang = min(max(tang, MIN_TILT_ANGLE), MAX_TILT_ANGLE);
    Serial.print(i);
    Serial.print(" - ");    ;    
    Serial.print(servo_pang);    
    Serial.print(" - ");
    Serial.println(servo_tang); 
    PANmotor_sweep(servo_pang, 10);
    TILTmotor_sweep(servo_tang, 10);
  }
}

void fade_laser() {
  Serial.println("~~~***FADE LASER UP/DOWN");
  for(int i = 0 ; i < 254; i++ ) {
    Serial.print(i);    
    analogWrite(PIN_LASER, i);
    delay(5);

  }
  for(int i = 255 ; i > 0; i--) {
    Serial.print(i);        
    analogWrite(PIN_LASER, i);
    delay(5);
  }
}



//---------------------------------------------------------//
//---------------------SERIAL COMMAND FXNS-----------------//
//---------------------------------------------------------//
void set_mode(int thisint) {    //-SETS THE MODE
  if (thisint <= MAX_MODE) {
    thisMode = thisint;
    Serial.print("~~~***NEW MODE-");
    Serial.println(thisMode);
    one_color_all(0,0,0);
    laserPOWER = 0;
    set_laser_power(laserPOWER);
  }
}

void set_laser_power(int thisint) {    //-SETS THE MODE
  //int lpower = min(thisint, 255);
  int lpower = min(max(thisint, 0), 255);
  analogWrite(PIN_LASER, lpower);
  Serial.print("~~~***LASER-");
  Serial.println(lpower);
  laserPOWER = lpower;
}


//---SWEEPS PAN MOTOR TO NEW ANGLE, CHECKS MIN/MAX VALUES
void BROKEN_PANmotor_sweep(int ang, int idelay) {
  //if (ang == servoPANpos) {return;}
  int oldang = min(max(servoPANpos, MIN_PAN_ANGLE), MAX_PAN_ANGLE);
  int newang = min(max(ang, MIN_PAN_ANGLE), MAX_PAN_ANGLE);
  int angdiff = newang - oldang;
  int np = negpos(angdiff);
  Serial.print("~~~***SETTING PAN FROM: ");
  Serial.print(oldang);
  Serial.print(" TO: ");
  Serial.print(newang);
  Serial.print(" : ");
  Serial.println(np);  
  if (np > 0){
    for(int i = oldang; i <= newang; i++){
      myservoPAN.write(i);
      delay(idelay);
    }
  }
  if (np < 0){
    for(int i = oldang; i >= newang; i--){        
      myservoPAN.write(i);
      Serial.println(i);
      delay(idelay);
    }
  }
  servoPANpos = newang; 
}

void PANmotor_sweep(int ang, int idelay) {
  int servo_ang = min(max(ang, MIN_PAN_ANGLE), MAX_PAN_ANGLE);
  myservoPAN.write(servo_ang);
  delay(idelay);  

}

void TILTmotor_sweep(int ang, int idelay) {
  int servo_ang = min(max(ang, MIN_TILT_ANGLE), MAX_TILT_ANGLE);
  myservoTILT.write(180-servo_ang);
  delay(idelay);  
}

void wiichucklaser_joycontrol() {
  //---PAN/TILT w/ JOY | LASER +/- w/ Z/C
  chuck.update();
    
  //---WIIMOTE JOYSTICK (X|LR, Y|UD, -100, 100
  int p_temp = motorVAL(chuck.readJoyX());
  int t_temp = motorVAL(chuck.readJoyY());
  PANmotor_sweep(p_temp, 1);
  TILTmotor_sweep(t_temp, 1);
  
  //---WIIMOTE BUTTONS    
  if (chuck.buttonZ) {
     laserPOWER -= 1;
     set_laser_power(laserPOWER);
  }
  if (chuck.buttonC) {
     laserPOWER += 1;
     set_laser_power(laserPOWER);        
  }

  delay(10);  
}

void wiichucklaser_gyrocontrol() {
  //---PAN/TILT w/ GYRO | LASER +/- w/ Z/C
  chuck.update();
  //---WIIMOTE PITCH/ROLL (MAKE THIS DO SOMETHING WITH LEDS?)
  //Serial.print(chuck.readRoll());
  //Serial.print(", ");  
  //Serial.print(chuck.readPitch());
  //Serial.print(", ");  
    
  //---WIIMOTE JOYSTICK (X|LR, Y|UD, -100, 100
  int p_temp = motorVAL(chuck.readRoll());
  int t_temp = motorVAL(chuck.readPitch());
  PANmotor_sweep(p_temp, 1);
  TILTmotor_sweep(t_temp, 1);
  
  //---WIIMOTE BUTTONS    
  if (chuck.buttonZ) {
     laserPOWER -= 1;
     set_laser_power(laserPOWER);
  }
  if (chuck.buttonC) {
     laserPOWER += 1;
     set_laser_power(laserPOWER);        
  }

  delay(10);  
}

//---CONVERT {-100, 100} to servo PAN/TILT
//---0, 100 = 90, 180
//---0,-100 = 0, 90
int motorVAL(int power){ 
  int powerOUT;
  float powerRAT = float(power)/100.0; 
  if (power >= 0) {
    powerOUT = 90 + powerRAT * 90;
  }
  else if (power < 0) {
    powerOUT = 90 - abs(powerRAT * 90);
  }  
  return powerOUT;
}



//---------------------------------------------------------//
//---------------------------MAIN--------------------------//
//---------------------------------------------------------//
void setup() {
  //---SETUP SERIAL OUTPUT
  Serial.begin(9600);
  softSerial.begin(9600);
  //Serial.println("|---BEGIN BLUEBOX ARDUINO SKETCH---");

  //---SETUP LASER
  pinMode(PIN_LASER, OUTPUT);

  //---SETUP INTERRUPT
  attachInterrupt(PIN_BUTTON_A, interruptHandler, FALLING);

  //---SETUP THERMOMETER
  sensor_thermo.begin();
  sensor_thermo.setResolution(0, TEMP_PRECISION);

  //---SETUP SERVOS
  myservoPAN.attach(PIN_SERVO_PAN);
  myservoTILT.attach(PIN_SERVO_TILT);

  //---SETUP AUDIO 
  //---DIGITAL/ANALOG
  
  //---SETUP LED STRIP
  FastSPI_LED.setLeds(NUM_LEDS);
  FastSPI_LED.setChipset(CFastSPI_LED::SPI_TM1809);
  FastSPI_LED.setPin(PIN_LED_STRIP);
  FastSPI_LED.setDataRate(LED_DATA_RATE);  
  FastSPI_LED.init();
  FastSPI_LED.start();
  led_array = (struct CRGB*)FastSPI_LED.getRGBData();  
  
  //---SETUP NUNCHUCK
  chuck.begin();
  chuck.update();
  //chuck.calibrateJoy();
}

void loop() {    
  //---PROCESS SOFTWARE SERIAL COMMANDS
  if (softSerial.available() > 0) {
    inbyte = softSerial.read();
    Serial.println(inbyte);
    if (inbyte == 109) { //---"m" - SET MODE
      thisarg = softSerial.parseInt();
      set_mode(thisarg);
    }
    if (inbyte == 97) { //---"a" - SET SERVO_PAN
      thisarg = softSerial.parseInt();
      PANmotor_sweep(thisarg, 10);
    }
    if (inbyte == 98) { //---"b" - SET SERVO_TILT
      thisarg = softSerial.parseInt();
      TILTmotor_sweep(thisarg, 10);
    }
    if (inbyte == 108) { //---"l" - SET LASER POWER
      thisarg = softSerial.parseInt();
      set_laser_power(thisarg);
    }

  }

  //---PROCESS SERIAL COMMANDS
  if (Serial.available() > 0) {
    inbyte = Serial.read();
    if (inbyte == 109) { //---"m" - SET MODE
      thisarg = Serial.parseInt();
      set_mode(thisarg);
    }
    if (inbyte == 97) { //---"a" - SET SERVO_PAN
      thisarg = Serial.parseInt();
      PANmotor_sweep(thisarg, 10);
    }
    if (inbyte == 98) { //---"b" - SET SERVO_TILT
      thisarg = Serial.parseInt();
      TILTmotor_sweep(thisarg, 10);
    }
    if (inbyte == 108) { //---"l" - SET LASER POWER
      thisarg = Serial.parseInt();
      set_laser_power(thisarg);
    }
  }
  
  //---EXECUTE MODES
  if (thisMode == 0){
    one_color_all(0,0,0);    
  }  
  if (thisMode == 1){
    VUPing();
  }
  if (thisMode == 2){
    VUThermo();
  }  
  if (thisMode == 3){
    VUAudio();
  }
  if (thisMode == 4){
    color_bounce(35);
  }
  if (thisMode == 5){
    rainbow_loop(15, 30);
  }
  if (thisMode == 6){
    wiichucklaser_gyrocontrol();
  }
  if (thisMode == 7){
    wiichucklaser_joycontrol();
  }    
  if (thisMode == 8){
    servo_sine_circle();
  }
  if (thisMode == 9){
    fade_laser();
  }   
}
