//-------------------------------------------------------------------//
//-------------------------------------------------------------------//
//------------ROS (ROBOT OPERATING SYSTEM) FIRMWARE FOR RYGEL--------//
//-------------------------------------------------------------------//
//-------------------------------------------------------------------//
//------------------------------NOTES--------------------------------//
//-SONAR = HC-SR04
//-IMU = MPU6050 | GY-521
//-PAN/TILT SERVOS | 9gram angle (0-180?ish)
//-MOTOR SERVOS | Xgram continuous rotation (0-180?ish)
//-DRIVE MOTORS | 4=RB(-) | 3=LB(-) | 2=RF(-) | 1=LF | 0-89 REVERSE / 91-180-FORWARD
//-------------------------------------------------------------------//
//-CANNOT USE sensor IMU or RANGE message types, probably that Arduino doesn't like Float64
//-------------------------------------------------------------------//
/////---------NEXT
//---make a published STRING for various info out, aka chatter publisher...
//---	battery status? voltage somehow- whatever...
//---	and use that to diagnose the MPU 6dof thing
//---	so not have to use the RAW values
//------------------LATER-------------------
//---zero out timer vars at some point!!!
//---sweep and limits on pan/tilt? drive? - no- do that on the rospy side so its non-blocking
//---might be an issue- using unsignedINt16 in msg
//---ADD LIMITS TO DRIVE MOTORS! did that...
//-------------------------------------------------------------------//

//---ROS INCLUDES
#include <ros.h>
#include <std_msgs/Empty.h>
#include <std_msgs/Int16.h>
#include <std_msgs/Float32.h>
#include <geometry_msgs/Point32.h>
#include <std_msgs/String.h>
#include <rygel/rygelIMURAW.h>

//---OTHER INCLUDES
#include <Servo.h>          //-FOR SERVOS
#include <NewPing.h>        //-FOR RANGEFINDERS
#include <Wire.h>           //-FOR FREEIMU
#include <I2Cdev.h>         //-FOR FREEIMU
#include <MPU6050.h>        //-FOR FREEIMU

//-------------------------------------------------------------------//
//---DEFINE PIN NUMBERS
#define BOARD_LED_PIN           13   //---ONBOARD LED PIN
#define MOTOR_ALT_PIN           12   //---ALTERNATE MOTOR PIN
#define MOTOR_PAN_PIN           10   //---PAN MOTOR PIN (10 on RYGEL)
#define MOTOR_TILT_PIN          11   //---TILT MOTOR PIN
#define MOTOR_DRIVE_LF_PIN      3    //---LEFT FRONT DRIVE MOTOR PIN
#define MOTOR_DRIVE_RF_PIN      5    //---RIGHT FRONT DRIVE MOTOR PIN
#define MOTOR_DRIVE_LB_PIN      6    //---LEFT BACK DRIVE MOTOR PIN
#define MOTOR_DRIVE_RB_PIN      9    //---RIGHT BACK DRIVE MOTOR PIN
#define SONAR_0_TRIGGER_PIN     A0   //---SONAR 0 TRIGGER (PIN 14)
#define SONAR_0_ECHO_PIN        A1   //---SONAR 0 ECHO (PIN 15)
#define SONAR_1_TRIGGER_PIN     A2   //---SONAR 1 TRIGGER (PIN 16)
#define SONAR_1_ECHO_PIN        A3   //---SONAR 1 ECHO (PIN 17)

//---DEFINE CONSTANTS
#define SONAR_MAX_DISTANCE      200  //---SONAR MAX DISTANCE
#define SONAR_SAMPLE_INTERVAL   4000 //---SONAR SAMPLING INTERVAL (4000 = 1/sec)
#define IMU_SAMPLE_INTERVAL     500  //---IMU SAMPLING INTERVAL
#define TILT_MIN                90   //---PAN/TILT MIN/MAX ANGLES
#define TILT_MAX                150
#define PAN_MIN                 60
#define PAN_MAX                 120
#define DRIVE_FWD_MAX           50   //---DRIVE MAX PERCENT POWER (0, 100
#define DRIVE_REV_MAX           -25  //---DRIVE MAX PERCENT POWER (0,-100)

//---DEFINE VARIABLES
long timerSONAR_0;                   //---SONAR_0 SAMPLING TIMERS
long timerSONAR_1;
long timerIMU;
char blah[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
int16_t ax, ay, az;                  //---IMU VARS
int16_t gx, gy, gz;



//-------------------------------------------------------------------//
//---CREATE SERVO MOTOR OBJECTS
Servo objMOTOR_ALT;
Servo objMOTOR_PAN;
//Servo objMOTOR_TILT;
Servo objMOTOR_DRIVE_LF;
Servo objMOTOR_DRIVE_RF;
Servo objMOTOR_DRIVE_LB;
Servo objMOTOR_DRIVE_RB;

//---CREATE ULTRASONIC SONAR OBJECTS
NewPing objSONAR_0(SONAR_0_TRIGGER_PIN, SONAR_0_ECHO_PIN, SONAR_MAX_DISTANCE);
NewPing objSONAR_1(SONAR_1_TRIGGER_PIN, SONAR_1_ECHO_PIN, SONAR_MAX_DISTANCE);

//---GYRO/ACC FREEIMU OBJECTS
MPU6050 objMPU;



//--------------------------------------------------------------------//
//-------------------------- COMMAND FXNS ----------------------------//
//--------------------------------------------------------------------//
void TOGGLE_LED_cb( const std_msgs::Empty& toggle_msg){
  digitalWrite(BOARD_LED_PIN, HIGH-digitalRead(BOARD_LED_PIN));
}
void MOTOR_ALT_servo_cb( const std_msgs::Int16& cmd_msg){
  objMOTOR_ALT.write(cmd_msg.data); //---SET SERVO ANGLE | 0-180  
}
void MOTOR_PAN_servo_cb( const std_msgs::Int16& cmd_msg){
  int pan_val = min(max(cmd_msg.data, PAN_MIN), PAN_MAX);  
  objMOTOR_PAN.write(pan_val);
}
//void MOTOR_TILT_servo_cb( const std_msgs::UInt16& cmd_msg){
//  int tilt_val = min(max(cmd_msg.data, TILT_MIN), TILT_MAX);
//  objMOTOR_TILT.write(tilt_val);
//}
void MOTOR_DRIVE_LF_servo_cb( const std_msgs::Int16& cmd_msg){
  int drive_val = motorVAL(cmd_msg.data);
  objMOTOR_DRIVE_LF.write(drive_val);
}
void MOTOR_DRIVE_RF_servo_cb( const std_msgs::Int16& cmd_msg){
  int drive_val = motorVAL(cmd_msg.data);
  objMOTOR_DRIVE_RF.write(drive_val);
}
void MOTOR_DRIVE_LB_servo_cb( const std_msgs::Int16& cmd_msg){
  int drive_val = motorVAL(cmd_msg.data);  
  objMOTOR_DRIVE_LB.write(drive_val);
}
void MOTOR_DRIVE_RB_servo_cb( const std_msgs::Int16& cmd_msg){
  int drive_val = motorVAL(cmd_msg.data);  
  objMOTOR_DRIVE_RB.write(drive_val);
}
int getSONAR_0(){        //---RETURN PING IN CM
  unsigned int ping_time = objSONAR_0.ping();
  return (ping_time / US_ROUNDTRIP_CM);
}
int getSONAR_1(){
  unsigned int ping_time = objSONAR_1.ping();
  return (ping_time / US_ROUNDTRIP_CM);
}




//-------------------------------------------------------------------//
//------------------------------ ROS --------------------------------//
//-------------------------------------------------------------------//
//---SETUP ROS NODE
ros::NodeHandle  nh;

//---DEFINE PUBLISHED MESSAGE TYPES
std_msgs::Float32 SONAR_0_MSG;
std_msgs::Float32 SONAR_1_MSG;
rygel::rygelIMURAW IMURAW_MSG;
std_msgs::String BLAH_MSG;

//---SETUP PUBLISHERS (PUBLISH SENSOR DATA)
ros::Publisher pubSONAR_0("SONAR_0", &SONAR_0_MSG);
ros::Publisher pubSONAR_1("SONAR_1", &SONAR_1_MSG);
ros::Publisher pubIMURAW("IMU_RAW", &IMURAW_MSG);
ros::Publisher pubBLAH("BLAH", &BLAH_MSG);

//---SETUP SUBSCRIPTIONS (SUBSCRIBE TO COMMANDS)
ros::Subscriber<std_msgs::Empty> subTOGGLE_LED("TOGGLE_LED", &TOGGLE_LED_cb );
ros::Subscriber<std_msgs::Int16> subMOTOR_ALT("MOTOR_ALT", MOTOR_ALT_servo_cb);
ros::Subscriber<std_msgs::Int16> subMOTOR_PAN("MOTOR_PAN", MOTOR_PAN_servo_cb);
//ros::Subscriber<std_msgs::UInt16> subMOTOR_TILT("MOTOR_TILT", MOTOR_TILT_servo_cb);
ros::Subscriber<std_msgs::Int16> subMOTOR_DRIVE_LF("MOTOR_DRIVE_LF", MOTOR_DRIVE_LF_servo_cb);
ros::Subscriber<std_msgs::Int16> subMOTOR_DRIVE_RF("MOTOR_DRIVE_RF", MOTOR_DRIVE_RF_servo_cb);
ros::Subscriber<std_msgs::Int16> subMOTOR_DRIVE_LB("MOTOR_DRIVE_LB", MOTOR_DRIVE_LB_servo_cb);
ros::Subscriber<std_msgs::Int16> subMOTOR_DRIVE_RB("MOTOR_DRIVE_RB", MOTOR_DRIVE_RB_servo_cb);


//--------------------------- OTHER FXNS ----------------------------//
//---CONVERT {-100, 100} to servo
//---0, 100 = 90, 0
//---0,-100 = 90, 180
int motorVAL(int power){ 
  int powerIN = min(max(power, DRIVE_REV_MAX), DRIVE_FWD_MAX);
  int powerOUT;
  float powerRAT = float(powerIN)/100.0;
  if (powerIN == 0) {
    powerOUT = 90;}  
  if (powerIN > 0) {
    powerOUT = 90 - (powerRAT * 90);}
  if (powerIN < 0) {
    powerOUT = (abs(powerRAT) * 90) + 90;}  

  return powerOUT;
}



//-------------------------------------------------------------------//
//-----------------------SETUP/LOOP FUNCTIONS------------------------//
//-------------------------------------------------------------------//
void setup() {
  nh.initNode();  
  
  nh.subscribe(subTOGGLE_LED);
  nh.subscribe(subMOTOR_ALT);  
  nh.subscribe(subMOTOR_PAN);
  //nh.subscribe(subMOTOR_TILT);    
  nh.subscribe(subMOTOR_DRIVE_LF);
  nh.subscribe(subMOTOR_DRIVE_RF);
  nh.subscribe(subMOTOR_DRIVE_LB);
  nh.subscribe(subMOTOR_DRIVE_RB);
  
  nh.advertise(pubSONAR_0);
  nh.advertise(pubSONAR_1);    
  nh.advertise(pubIMURAW); 
  nh.advertise(pubBLAH);
  
  objMOTOR_ALT.attach(MOTOR_ALT_PIN);            //---ATTACH SERVO OBJECTS
  objMOTOR_PAN.attach(MOTOR_PAN_PIN);            
  //objMOTOR_TILT.attach(MOTOR_TILT_PIN);
  objMOTOR_DRIVE_LF.attach(MOTOR_DRIVE_LF_PIN);
  objMOTOR_DRIVE_RF.attach(MOTOR_DRIVE_RF_PIN);
  objMOTOR_DRIVE_LB.attach(MOTOR_DRIVE_LB_PIN);
  objMOTOR_DRIVE_RB.attach(MOTOR_DRIVE_RB_PIN);  

  Wire.begin();                                  //---INIT IMU
  objMPU.initialize();
  
}



void loop() {  
  if (millis() > timerSONAR_0) { //---SONAR_0
    SONAR_0_MSG.data = getSONAR_0();
    pubSONAR_0.publish(&SONAR_0_MSG);
    timerSONAR_0 = millis() + SONAR_SAMPLE_INTERVAL;
  }
  if (millis() > timerSONAR_1) { //---SONAR_1
    SONAR_1_MSG.data = getSONAR_1();
    pubSONAR_1.publish(&SONAR_1_MSG);
    timerSONAR_1 = millis() + SONAR_SAMPLE_INTERVAL;
  }  
  if (millis() > timerIMU) { //---IMU-ROTATION
    objMPU.getMotion6(&ax, &ay, &az, &gx, &gy, &gz);
    IMURAW_MSG.header.stamp = nh.now();
    IMURAW_MSG.gx = gx;
    IMURAW_MSG.gy = gy;
    IMURAW_MSG.gz = gz;
    IMURAW_MSG.ax = ax;
    IMURAW_MSG.ay = ay;
    IMURAW_MSG.az = az;
    pubIMURAW.publish(&IMURAW_MSG);    
    timerIMU = millis() + IMU_SAMPLE_INTERVAL;    
  }
  

  nh.spinOnce();
  delay(1);

}


//---------------------------------------
//dtostrf(powerRAT, 1, 2, blah);
//itoa(powerOUT, blah, DEC);
//BLAH_MSG.data = blah;
//pubBLAH.publish( &BLAH_MSG );


